/**********************************/
/*                                */
/*  Copyright 2000, David Grant   */
/*                                */
/*  see LICENSE for more details  */
/*                                */
/**********************************/

long add_5206_register(void);
long adda_5206_register(void);
long addi_5206_register(void);
long addq_5206_register(void);
long addx_5206_register(void);
long and_5206_register(void);
long andi_5206_register(void);
long asx_5206_register(void);
long bcc_5206_register(void);
long btst_5206_register(void);
long clr_5206_register(void);
long cmp_5206_register(void);
long cmpa_5206_register(void);
long cmpi_5206_register(void);
long dc_5206_register(void);
long eor_5206_register(void);
long eori_5206_register(void);
long ext_5206_register(void);
long halt_5206_register(void);
long illegal_5206_register(void);
long jmp_5206_register(void);
long jsr_5206_register(void);
long lea_5206_register(void);
long link_5206_register(void);
long lsx_5206_register(void);
long move_5206_register(void);
long movec_5206_register(void);
long movem_5206_register(void);
long moveq_5206_register(void);
long movexr_5206_register(void);
long mulu_l_5206_register(void);
long mul_w_5206_register(void);
long neg_5206_register(void);
long negx_5206_register(void);
long nop_5206_register(void);
long not_5206_register(void);
long or_5206_register(void);
long ori_5206_register(void);
long pea_5206_register(void);
long rte_5206_register(void);
long rts_5206_register(void);
long scc_5206_register(void);
long stop_5206_register(void);
long sub_5206_register(void);
long suba_5206_register(void);
long subi_5206_register(void);
long subq_5206_register(void);
long subx_5206_register(void);
long swap_5206_register(void);
long trap_5206_register(void);
long trapf_5206_register(void);
long tst_5206_register(void);
long unlk_5206_register(void);
